/***********************************************************************
 * File: abacus.h
 * 	(Extracted from
 *	   "$Id: abacus.h,v 1.4 2009/06/05 19:15:45 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header file contains definitions of `abacus()' API.
 *
 * Copyright (C) 2006-2009 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _ABACUS_H_ /* { */
#define _ABACUS_H_
#ifdef __cplusplus
extern "C" {
#endif

/* abacus needs the bellow system include files */
#include <unistd.h>
#include <limits.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <errno.h>


/* abacus can calculate 18 digits for integer and fractional. thus,
 * maximum length of result string that abacus outputs is 38 bytes
 */
#define MAX_Digits		18
#define SZ_Numeral_str	(1 + MAX_Digits + 1 + MAX_Digits + 1)


/* definitions for operand qualifiers for abacus */
extern char *register_0;
#define Register0	(void *)(register_0)
#define Memory(n)	(void *)(n)
#define Value(str)	(void *)(str)

typedef enum how_to_rounding_e {
	ROUND_up, ROUND_even, ROUND_down
} How_to_rounding_e;
#define Round_up	(void *)(ROUND_up)
#define Round_even	(void *)(ROUND_even)
#define Round_down	(void *)(ROUND_down)


/* definitions of opcode of abacus */
typedef enum opcode_e {
	/* [notation]
	 *	Nop
	 * abacus() sets `pc' to indicate the next command.
	 */
	  Nop_op
	  #define Nop {									\
	  	Nop_op										\
		, {	NULL, NULL, NULL, NULL, NULL	}		\
	  } 

	/* [notation]
	 *	Ret(	retcode					)
	 * abacus() returns immediately. the return-value will be
	 * `Retcode'.
	 */
	, Ret_op
	  #define Ret(Retcode) {						\
	  	Ret_op										\
		, {	(void *)(Retcode)						\
			, NULL, NULL, NULL, NULL				\
		}											\
	  } 

	/* [notation]
	 *	Set(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<Register0|Memory(n)|Strings(src)>	)
	 * abacus() transfers data of `Src' to `Dst'.
	 */
	, Set_op
	  #define Set(Dst,Src) {						\
	  	Set_op										\
		, {	(void *)Dst, (void *)Src				\
			, NULL, NULL, NULL						\
		}											\
	  }

	/* [notation]
	 *	Jmp(	<skip-steps>				)
	 * abacus() sets `pc' to `Steps'. `Steps' never be zero.
	 */
	, Jmp_op
	  #define Jmp(Steps) {							\
	  	Jmp_op										\
		, {	(void *)(Steps)							\
			, NULL, NULL, NULL, NULL				\
		}											\
	  } 

	/* [notation]
	 *	Cmp(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<Register0|Memory(n)|Strings(src)>
	 *	,	<less-skip-steps>
	 *	,	<equals-skip-steps>
	 *	,	<greater-skip-steps>			)
	 * abacus() compares data of `A' and `B', and set `pc' to
	 * `LeSt', `EqSt' or `GrSt', according to the result of
	 * comparing. `LeSt', `EqSt' and `GrSt' never be zero.
	 */
	, Cmp_op
	  #define Cmp(A,B,LeSt,EqSt,GrSt) {				\
	  	Cmp_op										\
		, {	(void *)A, (void *)B					\
			, (void *)(LeSt)						\
			, (void *)(EqSt)						\
			, (void *)(GrSt)						\
		}											\
	  }

	/* [notation]
	 *	Sub(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<Register0|Memory(n)|Strings(src)>	)
	 * abacus() subtracts `subtrahend' from `minuend',
	 * and sets the result to `Register0'.
	 */
	, Sub_op
	  #define Sub(Minuend,Subtrahend) {				\
	  	Sub_op										\
		, {	(void *)Minuend, (void *)Subtrahend		\
			, NULL, NULL, NULL						\
		}											\
	  }

	/* [notation]
	 *	Add(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<Register0|Memory(n)|Strings(src)>	)
	 * abacus() adds `Augend' and `Addend', and set the result to
	 * Register0.
	 */
	, Add_op
	  #define Add(Augend,Addend) {					\
	  	Add_op										\
		, {	(void *)Augend, (void *)Addend			\
			, NULL, NULL, NULL						\
		}											\
	  }

	/* [notation]
	 *	Mul(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<Register0|Memory(n)|Strings(src)>	)
	 * abacus() multiplies `Multiplicand' by `Multiplier',
	 * and set the result to `Register0'.
	 */
	, Mul_op
	  #define Mul(Multiplicand,Multiplier) {		\
	  	Mul_op										\
		, {	(void *)Multiplicand					\
			, (void *)Multiplier					\
			, NULL, NULL, NULL						\
		}											\
	  }

	/* [notation]
	 *	Div(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<Register0|Memory(n)|Strings(src)>
	 *	,	<NO_Remainder|Register0|Memory(n)>	)
	 * abacus() divides `Dividend' with `Divisor' and set the result
	 * to `Register0'.
	 * digits of `Divisor' must be less than 19.
	 *	(ex. -999999999999999999,   -999999999999999998,  ...
	 *	   , -99999999999999999.9,  -99999999999999999.8, ...
	 *	   , -1.00000000000000001,  -1,
	 *	   , -.99999999999999999,   -.999999999999999998, ...
	 *	   , -.00000000000000002,   -.00000000000000001,  0
	 *	   ,  .00000000000000001,    .000000000000000002, ...
	 *	   ,  .99999999999999999,    .999999999999999999,
	 *	   ,  1,		    1.00000000000000001, ...
	 *	   ,  9999999999999999.8,   99999999999999999.9, ...
	 *	   ,  99999999999999998,    999999999999999999
	 *	)
	 * If `Remainder' is `Register0', abacus() calculates the
	 * reminder and set it register#0.  If `Remainder' is
	 * `Memory(N)' abacus() set the remainder the memory slot.
	 * If `Remainder' isn't `NO_Remainder', abacus() doesn't set
	 * the remainder of `Dividend' and `Divisor'.  Otherwise,
	 * abacus() never set remainder.
	 */
	, Div_op
	  #define NO_Remainder (void *)(-1)
	  #define Div(Dividend,Divisor,Remainder) {		\
	  	Div_op										\
		, {	(void *)Dividend, (void *)Divisor		\
			, (void *)Remainder, NULL, NULL			\
		}											\
	  }

	/* [notation]
	 *	Rou(	<Register0|Memory(n)|Strings(dst)>
	 *	,	<exponents>
	 *	,	<Round_up|Round_even|Round_down>	)
	 * abacus() rounds `Val' at `Exp'-exponent (`How' indicates
	 * method of rounding (`Round_up', `Round_even' or `Round_down')
	 * and set the result to `Register0'.
	 */
	, Rou_op
	  #define Rou(Val,Exp,How) {					\
	  	Rou_op										\
	  	, {	(void *)Val, (void *)Exp, (void *)How	\
			, NULL, NULL							\
		}											\
	  }

} Opcode_e;


/* definitions list of commands for abacus */
typedef struct command_t {
	Opcode_e	opcode; /* specify the opcode */
	#define		MAX_N_operands	5
	void		*operands[MAX_N_operands];
} Command_t;


/* API of abacus architecture */
extern int abacus(char		*result
				, size_t	result_sz
				, size_t	int_digits
				, size_t	fra_digits
				, size_t	memories_n
				, size_t	commands_n
				, Command_t	*commands
				, size_t	*pc
				);
#	define NORMAL_END	0 /* abacus() completely calculated */

#ifdef __cplusplus
}
#endif
#endif /* } !defined(_ABACUS_H_) */
