/***********************************************************************
 * File: aggregate.h
 * 	(Extracted from
 *	   "$Id: aggregate.h,v 1.15 2009/07/08 12:02:55 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header file contains definitions of `aggregate()' API.
 *
 * Copyright (C) 2009 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _AGGREGATE_H_ /* { */
#define _AGGREGATE_H_
#ifdef __cplusplus
extern "C" {
#endif

/* abacus needs the bellow system include files */
#include <unistd.h>
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <memory.h>
#include <errno.h>
#include <math.h>

/* maximum length of maximum bit-width integer */
#define MAX_LEN_OF_MAX_BIT_WIDTH_INTEGER	(21)

/* precision of result of `logl(3M)' */
#define PRECISON_OF_LOGL_MATH_LIB			(15)

/* maximum value of `unsigned long' */
#define MAX_ULONG_STR						"4294967295"

/* definitions of ope-code for `aggregate_numbers()' API */
typedef enum aggregate_e_opcode {
	  Aggregate_e_opcode_add	/* add the numbers */
	, Aggregate_e_opcode_sub	/* subtract the numbers */
	, Aggregate_e_opcode_mul	/* multiplies the numbers */
	, Aggregate_e_opcode_div	/* divide the numbers */
	, Aggregate_e_opcode_n
} Aggregate_e_opcode;

/* API: aggregate the numeral strings
 * this returns address of the allocated result buffer.
 */
/*@null@*/ extern char *aggregate_numbers(Aggregate_e_opcode	opcode
										, const char			**numerals
										, size_t				n_numerals
										, ssize_t				scale
										);

/* API: release the buffer that is allocated by `aggregate_numbers()' */
extern int destroy_aggregate_numbers(char	*aggregate_numbers_result);

/* API: compare two numeral strings */
/*@null@*/ extern int *compare_numbers(const char		*a_str
										, const char	*b_str
										, int			*result
										);

/* API: predication: is the numeral string zero or not? */
extern int is_number_zero(const char	*num);

/* API: predication: is the numeral string one or not? */
extern int is_number_one(const char	*num);

/* API: duplicate the numeral string (allocate another new string) */
/*@null@*/ extern char *duplicate_numbers(const char	*num);

/* API: round the numeral string */
typedef enum aggregate_e_round_how {
	  Aggregate_e_round_even	= 0	/* rounding even					*/
	, Aggregate_e_round_up			/* rounding up						*/
	, Aggregate_e_round_down		/* rounding down					*/
	, N_Aggregate_e_round_how		/* number of rounding directives	*/
} Aggregate_e_round_how;

/*@null@*/ extern char *round_numbers(Aggregate_e_round_how	how
									, const char			*num
									, ssize_t				round_scale
									);

/* API: absolute the numeral string */
/*@null@*/ extern char *absolute_numbers(const char	*num
										, ssize_t	scale
										);

/* API: make zero */
/*@null@*/ extern char *make_zero_numbers(ssize_t	scale);

/* API: make power of ten */
/*@null@*/ extern char *make_power_of_ten(ssize_t	scale);

/* API: shift the numeral string */
/*@null@*/ extern char *shift_numbers(const char	*num, ssize_t	scale);

/* API: the greatest common divisor (for rational numbers) */
/*@null@*/ extern char *gcd_numbers(const char	*n, const char	*m);

/* API: the least common multiple (for rational numbers) */
/*@null@*/ extern char *lcm_numbers(const char	*n, const char	*m);

/* API: extraction square root	*/
/*@null@*/ extern char *sqrt_numbers(const char	*num, ssize_t	scale);

/* API: exponent of real numbers */
/*@null@*/ extern char	*exponent_numbers(const char	*num
										, ssize_t	scale
										);

/* API: Napierian logarithm for real numbers */
/*@null@*/ extern char *ln_numbers(const char	*num, ssize_t	scale);

/* API: logarithm for real numbers */
/*@null@*/ extern char *log_numbers(const char		*num
									, const char	*base
									, ssize_t		scale
									);

/* API: power of real numbers (x^y) */
/*@null@*/ extern char *power_of_numbers(const char		*x
										, const char	*y
										, ssize_t		scale
										);

/* API: n-th root of real numbers (`result'^y = x) */
/*@null@*/ extern char *root_of_numbers(const char		*x
										, const char	*y
										, ssize_t		scale
										);

/* API: predicate: the string means a numeral string? */
extern int is_numeral_string(const char	*str);

/* API: predicate: the numeral string is negative? */
extern int is_negative_number(const char	*str);

/* API: predicate: the numeral string is positive? */
extern int is_positive_number(const char	*str);

/* API: predicate: the numeral string means integer value? */
extern int is_integer_number(const char	*str);

/* API: predicate: the numeral string means an odd integer? */
extern int is_odd_number(const char	*str);

/* API: predicate: the numeral string means an even integer? */
extern int is_even_number(const char	*str);

/* API: ceil of numbers */
/*@null@*/ extern char *ceil_numbers(const char	*num);

/* API: floor of numbers */
/*@null@*/ extern char *floor_numbers(const char	*num);

/* API: invert the sign of numerals strings */
/*@null@*/ extern char	*invert_sign_of_numbers(const char	*num);

#ifdef __cplusplus
}
#endif
#endif /* } !defined(_AGGREGATE_H_) */
