/***********************************************************************
 * File: aggregate_test.c
 * 	(Extracted from
 *	   "$Id: aggregate_test.c,v 1.17 2009/07/08 12:02:55 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C source file implements test program for `aggregate()' API.
 *
 * Copyright (C) 2009 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#include "aggregate.h" /* definitions of `aggregate' API-s */

/* test program for `aggregate.c' API-s */
int main(int /*@unused@*/ ac, char /*@unused@*/ **av)
{
	static char	*codes	= "z+-. 09";
	static struct round_test_t {
		char	*str;
		ssize_t	begin_scale, end_scale;
	}							round_tests[] =
											{ { "-94.95"	, 3,	-2 }
											, { "-9.95"		, 3,	-1 }
											, { "-.95"		, 3,	 0 }
											, { "-0.0"		, 3,	-1 }
											, { "-0"		, 3,	-1 }
											, { "-.0"		, 3,	 0 }
											, { ".0"		, 3,	 0 }
											, { "0"			, 3,	-1 }
											, { "0.0"		, 3,	-1 }
											, { ".95"		, 3,	 0 }
											, { "9.95"		, 3,	-1 }
											, { "94.95"		, 3,	-2 }
											};
	static char					*strs[]	=	{ "-9999"
											, "-999 "
											, "-99.1"
											, "-99.0"
											, "-99  "
											, "-9.1 "
											, "-9.01"
											, "-9.0 "
											, "-0.1 "
											, "-.01 "
											, "-.001"
											, "-.000"
											, ""
											, "0.000"
											, "00.00"
											, "  0.0"
											, " .001"
											, "+.010"
											, " +.1 "
											, "   9."
											, " 9.01"
											, "  9.1"
											, "  +99"
											, " 99.1"
											, " 999 "
											, " 9999"
											};
	static char					*integer_part_formating[] =
												{ "12345"
												, "0.00006789"
												, "0.0000000011131719"
												};
	static Aggregate_e_opcode	opcodes[] =	{ Aggregate_e_opcode_add
											, Aggregate_e_opcode_sub
											, Aggregate_e_opcode_mul
											, Aggregate_e_opcode_div
											};
	static char					*signs[]	= { "-", "" };
	static char					*numerals[] =
											{ "9999999999999999999999999"
											, "9"
											, ""
											};
	static char					*sums[] =	{ "10", "9", "8", "7", "6"
											,  "5", "4", "3", "2", "1"
											};
	static struct gcd_params_t {
				char	*x, *y;
			}					gcds[] =	{ { "5",		"3"		}
											, { "3",		"6"		}
											, { "3",		"-6"	}
											, { "-6",		"3"		}
											, { "-6",		"-3"	}
											, { "0.06" ,	"0.3"	}
											, { "-0.03",	"0.6"	}
											, { "-0.06",	"-3"	}
											};
	static char					*power_of_xs[] =	{ "-10"
													, "3.14"
													}
								, *power_of_ys[] =
												{ "-21.0"
												, "0.04328279790196590079"
												  "77297661521998631121"
												  "85634692931816811429"
												  "52643786903968199597"
												  "00511753867456777947"
												};
	static char					*lns[] =	{ "0"
											, "1"
											, "0.04328279790196590079"
											  "77297661521998631121"
											  "85634692931816811429"
											  "52643786903968199597"
											  "00511753867456777947"
											};
	static char					*exponents[] =	{ "-3.14"
												, "0"
												, "1"
												};
	static char					*squares[] =	{ "5630738.132"
												, "10000"
												, "0.12345"
												, "0.0025"
												, "0"
												, "-1"
												};
	static struct roots_t {
			char	*x, *y;
		}						roots[] =	{ {	  "999", "0"		}
											, {	   "-2", "999"		}
											, {		"0", "-999"		}
											, {		"0", "3.14"		}
											, {		"1", "2"		}
											, {	"10000", "2"		}
											, {	 "3.14", "-2.71"	}
											};
	static char					*statistics1[] =	{ "-3"
													, "-0.5"
													, "0"
													, "5.5"
													};
	Aggregate_e_round_how		round_how;
	#define						Aggregate_numerals_sz					\
												(1 + 1 + 25 + 1 + 25 + 1)
	#define						N_numerals		(2)
	char						a_str[6], b_str[6]
								, numeral_strs[N_numerals][
												Aggregate_numerals_sz
											]
								, *numeral_strps[N_numerals]
								, *retstrp
								;
	ssize_t						scales[] =	{ -10, 0, 25, 50, 100 }
								, scale, round_scale
								;
	size_t						len, i, j, k, l, m, n, o, h
								, gprof_cnt, gprof_iter_max
								;
	int							*retval, result;

																		
	for (gprof_iter_max = (size_t)((ac == 2 && strcmp(av[1], "-P") == 0)?
																400: 1
									)
		, gprof_cnt = 0
	; gprof_cnt < gprof_iter_max
	; ++gprof_cnt
	) {	/* iterate the unit test for gprof profiling... */

	if ((ac == 2)
	&& (strcmp(av[1], "-Log") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for logarithm API... */
		fprintf(stderr
			,	"{{{ tests for logarithm_numbers()...\n"
			)
		; (void)fflush(stdout)
		;

		for (i = 0; i < (sizeof(roots) / sizeof(roots[0])); ++i) {
		for (j = 0; j < ((sizeof(scales) / sizeof(scales[0])) - 1); ++j) {
			fprintf(stderr, "log_numbers(\"%s\", \"%s\", %ld) returns "
					, roots[i].y, "3.14", (long)scales[j]
					)
			; (void)fflush(stderr)
			; errno	= 0
			;
			retstrp	= log_numbers(roots[i].y, "3.14", scales[j])
			;
			if (retstrp == NULL) {
				fprintf(stderr, "%p (errno=%d).\n", retstrp, errno)
				; (void)fflush(stderr)
				;
			}
			else {
				fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)fflush(stderr)
				; (void)destroy_aggregate_numbers(retstrp)
				;
			}
		}}

		fprintf(stderr
			,	"}}} tests for logarithm_numbers() finished!\n"
			)
		; (void)fflush(stdout)
		;
		if (strcmp(av[1], "-Log") == 0) {	exit(EXIT_SUCCESS);	}
	} /* test for logarithm API... */

	if ((ac == 2)
	&& (strcmp(av[1], "-Root") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for n-th-root API... */
		fprintf(stderr
			,	"{{{ tests for root_of_numbers()...\n"
			)
		; (void)fflush(stdout)
		;

		for (i = 0; i < (sizeof(roots) / sizeof(roots[0])); ++i) {
		for (j = 0; j < ((sizeof(scales) / sizeof(scales[0])) - 1); ++j) {
			fprintf(stderr, "root_of_numbers(\"%s\", \"%s\", %ld) returns "
					, roots[i].x, roots[i].y, (long)scales[j]
					)
			; (void)fflush(stderr)
			; errno	= 0
			;
			retstrp	= root_of_numbers(roots[i].x, roots[i].y, scales[j])
			;
			if (retstrp == NULL) {
				fprintf(stderr, " %p (errno=%d).\n", (void *)retstrp, errno)
				; (void)fflush(stderr)
				;
			}
			else {
				fprintf(stderr, " \"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)fflush(stderr)
				; (void)destroy_aggregate_numbers(retstrp)
				;
			}
		}}

		fprintf(stderr
			,	"}}} tests for root_of_numbers() finished!\n"
			)
		; (void)fflush(stdout)
		;
		if (strcmp(av[1], "-Root") == 0) {	exit(EXIT_SUCCESS);	}
	} /* test for n-th-root API finished! */

	if ((ac == 2)
	&& (strcmp(av[1], "-Floor") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for floor, ceil, invert-sign API-s... */
		fprintf(stderr
			,	"{{{ tests for ceil_numbers(), floor_numbers() and "
				"invert_sign_of_numbers()...\n"
			)
		; (void)fflush(stdout)
		;

		for (i = 0; i < (sizeof(statistics1) / sizeof(statistics1[0])); ++i)
		{
			fprintf(stderr, "ceil_numbers(\"%s\") returns ", statistics1[i])
			; (void)fflush(stderr)
			; errno	= 0
			;
			retstrp	= ceil_numbers(statistics1[i]);
			if (retstrp == NULL) {
				fprintf(stderr, "%p (errno=%d).\n", retstrp, errno)
				; (void)fflush(stderr)
				;
			}
			else {
				fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)fflush(stderr)
				; (void)destroy_aggregate_numbers(retstrp); retstrp = NULL
				;
			}

			fprintf(stderr, "floor_numbers(\"%s\") returns "
					, statistics1[i]
					)
			; (void)fflush(stderr)
			; errno	= 0
			;
			retstrp	= floor_numbers(statistics1[i]);
			if (retstrp == NULL) {
				fprintf(stderr, "%p (errno=%d).\n", retstrp, errno)
				; (void)fflush(stderr)
				;
			}
			else {
				fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)fflush(stderr)
				; (void)destroy_aggregate_numbers(retstrp); retstrp = NULL
				;
			}

			fprintf(stderr, "invert_sign_of_numbers(\"%s\") returns "
					, statistics1[i]
					)
			; (void)fflush(stderr)
			; errno	= 0
			;
			retstrp	= invert_sign_of_numbers(statistics1[i]);
			if (retstrp == NULL) {
				fprintf(stderr, "%p (errno=%d).\n", retstrp, errno)
				; (void)fflush(stderr)
				;
			}
			else {
				fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)fflush(stderr)
				; (void)destroy_aggregate_numbers(retstrp); retstrp = NULL
				;
			}
		}

		fprintf(stderr
			,	"}}} tests for ceil_numbers(), floor_numbers() and "
				"invert_sign_of_numbers() finished!\n"
			)
		; (void)fflush(stdout)
		;
		if (strcmp(av[1], "-Floor") == 0) {	exit(EXIT_SUCCESS);	}
	}	/* test for floor, ceil, invert-sign API-s finished! */

	if ((ac == 2)
	&& (strcmp(av[1], "-GCD") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for ln_numbers()... */
		fprintf(stderr
			, "{{{ tests for gcd_numbers() and lcm_numbers()...\n"
			)
		; (void)fflush(stdout)
		;

		for (i = 0; i < (sizeof(gcds) / sizeof(gcds[0])); ++i) {
			fprintf(stderr, "gcd_numbers(\"%s\", \"%s\") returns "
					, gcds[i].x, gcds[i].y
					)
			; (void)fflush(stderr)
			; errno	= 0
			; retstrp	= gcd_numbers(gcds[i].x, gcds[i].y)
			;
			if (retstrp == NULL) {
				fprintf(stderr, "%p (errno=%d).\n", retstrp, errno);
			}
			else {
				fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)destroy_aggregate_numbers(retstrp)
				;
			}
			(void)fflush(stderr);

			fprintf(stderr, "lcm_numbers(\"%s\", \"%s\") returns "
					, gcds[i].x, gcds[i].y
					)
			; (void)fflush(stderr)
			; errno	= 0
			; retstrp	= lcm_numbers(gcds[i].x, gcds[i].y)
			;
			if (retstrp == NULL) {
				fprintf(stderr, "%p (errno=%d).\n", retstrp, errno);
			}
			else {
				fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
				; (void)destroy_aggregate_numbers(retstrp)
				;
			}
			(void)fflush(stderr);
		}

		fprintf(stderr
			, "}}} tests for gcd_numbers() and lcm_numbers() finished!\n"
			)
		; (void)fflush(stdout)
		;
		if (strcmp(av[1], "-GCD") == 0) {	exit(EXIT_SUCCESS);	}
	}	/* test for ln_numbers() finished. */

	if ((ac == 2)
	&& (strcmp(av[1], "-Pow") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for ln_numbers()... */
		fprintf(stderr
			, "{{{ tests for power_of_numbers()...\n"
			)
		; (void)fflush(stdout)
		;

		for (i = 0
		; i < (sizeof(power_of_xs)/ sizeof(power_of_xs[0]))
		; ++i
		) {
			for (j = 0
			; j < ((sizeof(scales) / sizeof(scales[0])) - 1)
			; ++j
			) {
				fprintf(stderr
						, "power_of_numbers(\"%s\", \"%s\", %ld) returns "
						, power_of_xs[i], power_of_ys[i], (long)scales[j]
						)
				; (void)fflush(stderr)
				; errno		= 0
				; retstrp	= power_of_numbers(power_of_xs[i]
											, power_of_ys[i]
											, scales[j]
											)
				;
				if (retstrp == NULL) {
					fprintf(stderr, "%p (errno=%d).\n", retstrp, errno)
					; (void)fflush(stdout)
					;
				}
				else {
					fprintf(stderr, "\"%s\"(len=%lu) (errno=%d).\n"
							, retstrp, (unsigned long)strlen(retstrp), errno
							)
					; (void)fflush(stderr)
					; (void)destroy_aggregate_numbers(retstrp)
					;
				}
			}
		}
		fprintf(stderr
			, "}}} tests for power_of_numbers() finished!\n"
			)
		; (void)fflush(stdout)
		;

		if (strcmp(av[1], "-Pow") == 0) {	exit(EXIT_SUCCESS);	}
	}	/* test for ln_numbers() finished! */

	if ((ac == 2)
	&& (strcmp(av[1], "-L") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for ln_numbers()... */
		fprintf(stderr
			, "{{{ tests for ln_numbers()...\n"
			)
		; (void)fflush(stdout)
		;
		for (i = 0; i < (sizeof(lns) / sizeof(lns[0])); ++i)
		{
			for (j = 0; j < (sizeof(scales) / sizeof(scales[0])); ++j) {
				fprintf(stderr, "ln_numbers(\"%s\", %ld) returns "
						, lns[i], (long)scales[j]
						)
				; (void)fflush(stderr)
				; retstrp = ln_numbers(lns[i], scales[j])
				;
				if (retstrp == NULL) {
					fprintf(stderr, "%p (errno=%d)"
							, (void *)retstrp, errno
							);
				}
				else {
					fprintf(stderr, "\"%s\"(len=%lu) (errno=%d)"
						, retstrp, (unsigned long)strlen(retstrp), errno
						)
					; (void)destroy_aggregate_numbers(retstrp)
					;
				}
				fprintf(stderr, "\n"); (void)fflush(stderr);
			}
		}
		fprintf(stderr
			, "}}} tests for ln_numbers() finished!\n"
			)
		; (void)fflush(stdout)
		;

		if (strcmp(av[1], "-L") == 0) {	exit(EXIT_SUCCESS);	}
	} /* ...test for ln_numbers() */

	if ((ac == 2)
	&& (strcmp(av[1], "-E") == 0 || strcmp(av[1], "-P") == 0)
	) {	/* test for elementary functions... */
		fprintf(stderr, "{{{ tests for exponent_numbers()...\n")
		; (void)fflush(stdout)
		;
		for (i = 0; i < (sizeof(exponents) / sizeof(exponents[0])); ++i)
		{
			for (j = 0; j < (sizeof(scales) / sizeof(scales[0])); ++j) {
				fprintf(stderr
						, "exponents_numbers(\"%s\", %ld) returns "
						,		exponents[i], (long)scales[j]
						);
				; (void)fflush(stderr)
				; retstrp	= exponent_numbers(exponents[i], scales[j])
				;
				if (retstrp == NULL) {
					fprintf(stderr, "%p (errno=%d)"
							,		(void *)retstrp, errno
							)
					; (void)fflush(stderr)
					;
				}
				else {
					fprintf(stderr, "\"%s\"(len=%lu) (errno=%d)"
						, retstrp, (unsigned long)strlen(retstrp)
						, errno
						)
					; (void)destroy_aggregate_numbers(retstrp)
					;
				}
				fprintf(stderr, "\n"); (void)fflush(stderr);
			}
		}
		fprintf(stderr, "}}} tests for exponent_numbers() finished!\n")
		; (void)fflush(stdout)
		;

		fprintf(stderr, "\n"); (void)fflush(stderr);

		fprintf(stderr, "{{{ tests for sqrt_numbers()...\n")
		; (void)fflush(stdout)
		;
		for (i = 0; i < (sizeof(squares) / sizeof(squares[0])); ++i) {
			for (j = 0; j < (sizeof(scales) / sizeof(scales[0])); ++j) {
				fprintf(stderr,	"sqrt_numbers(\"%s\", %ld) returns "
						,		squares[i], (long)scales[j]
						)
				; (void)fflush(stderr)
				; retstrp	= sqrt_numbers(squares[i], scales[j])
				;
				if (retstrp == NULL) {
					fprintf(stderr, "%p (errno=%d)"
							,		(void *)retstrp, errno
							);
				}
				else {
					fprintf(stderr, "\"%s\"(len=%lu) (errno=%d)"
						, retstrp, (unsigned long)strlen(retstrp)
						, errno
						)
					; (void)destroy_aggregate_numbers(retstrp)
					;
				}
				fprintf(stderr, "\n"); (void)fflush(stderr);
			}
		}
		fprintf(stderr, "}}} tests for sqrt_numbers() finished!\n")
		; (void)fflush(stdout)
		;

		fprintf(stderr, "\n"); (void)fflush(stderr);

		if (strcmp(av[1], "-E") == 0) {	exit(EXIT_SUCCESS);	}
	} /* ...test for elementary functions */

	fprintf(stderr, "{{{ tests for round_numbers()...\n")
	; (void)fflush(stdout)
	;
	for (i = 0; i < (sizeof(round_tests) / sizeof(round_tests[0])); ++i) {
	for (round_scale = round_tests[i].begin_scale
	; round_scale >= round_tests[i].end_scale
	; --round_scale
	) {
	for (round_how = Aggregate_e_round_even
	; round_how <= N_Aggregate_e_round_how
	; ++round_how
	) {
		fprintf(stderr,	"round_numbers(%s(=%d), [%s](len=%lu), %ld) "
						, ((round_how == Aggregate_e_round_even)?
											"Aggregate_e_round_even"
							: ((round_how == Aggregate_e_round_up)?
											"Aggregate_e_round_up"
							: ((round_how == Aggregate_e_round_down)?
											"Aggregate_e_round_down"
							:				"(unknown)"
							)))
						, (int)round_how
						, round_tests[i].str
						, (unsigned long)strlen(round_tests[i].str)
						, (long)round_scale
						)
		; (void)fflush(stderr)
		; retstrp = NULL; errno = 0
		;
		retstrp	= round_numbers(round_how
								, round_tests[i].str
								, round_scale
								);
		if (retstrp == NULL) {
			fprintf(stderr,	" NULL (%p, errno=%d) ***ERROR***\n"
							, retstrp, errno
							)
			; (void)fflush(stderr)
			;
		}
		else {
			fprintf(stderr,	" [%s](len=%lu)\n"
							, retstrp, (unsigned long)strlen(retstrp)
							)
			; (void)fflush(stderr)
			;

			(void)destroy_aggregate_numbers(retstrp); retstrp = NULL;
		}
	}
	}
	}
	fprintf(stderr, "}}} tests for round_numbers() finished!\n")
	; (void)fflush(stdout)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);

	fprintf(stderr, "{{{ tests for is_number_zero() and "
					"duplicated_numbers()...\n"
					)
	; (void)fflush(stdout)
	;
	for (i = 0; i < (sizeof(strs) / sizeof(strs[0])); ++i) {
		fprintf(stderr,	"strs[%lu]=[%s](len=%lu) is "
						, (unsigned long)i, strs[i]
						, (unsigned long)strlen(strs[i])
						)
		; (void)fflush(stderr)
		; errno	= 0
		;
		result	= is_number_zero(strs[i]);
		fprintf(stderr,	"%s"
						, ((result < 0)?		"not a number"
							: ((result == 1)?	"zero"
							:					"not zero"
							))
						);
		; (void)fflush(stderr)
		; fprintf(stderr,	" (is_number_zero() returns %d (errno=%d))\n"
						, result, errno
						)
		; (void)fflush(stderr)
		;
		if (result >= 0) {
			char	*duplicated = NULL;

			errno = 0
			;
			duplicated = duplicate_numbers(strs[i]);

			if (duplicated == NULL) {
				fprintf(stderr,	"***ERROR*** duplicated_numbers( "
								"strs[%lu]=[%s](len=%lu) ) "
								"returns error (%p: errno=%d)!\n"
								, (unsigned long)i, strs[i]
								, (unsigned long)strlen(strs[i])
								, (void *)duplicated, errno
								)
				; (void)fflush(stderr)
				;
			}
			else {
				if (strcmp(duplicated, strs[i]) != 0) {
					fprintf(stderr,	"***FATAL*** duplicated_numbers( "
									"strs[%lu]=[%s](len=%lu) ) "
									"returns differ string "
									"[%s]@%p(len=%lu)!\n"
									, (unsigned long)i, strs[i]
									, (unsigned long)strlen(strs[i])
									, duplicated, (void *)duplicated
									, (unsigned long)strlen(duplicated)
									)
					; (void)fflush(stderr)
					;
				}
				else {
					fprintf(stderr,	"duplicated_numbers( "
									"strs[%lu]=[%s](len=%lu) ) "
									"returns the same string "
									"[%s]@%p(len=%lu).\n"
									, (unsigned long)i, strs[i]
									, (unsigned long)strlen(strs[i])
									, duplicated, (void *)duplicated
									, (unsigned long)strlen(duplicated)
									)
					; (void)fflush(stderr)
					;
				}

				/* release the duplicated string that is allocated */
				(void)destroy_aggregate_numbers(duplicated)
				; duplicated = NULL
				;
			}
		}
	}
	fprintf(stderr, "}}} tests for is_number_zero() and "
					"duplicated_numbers() finished!\n"
					)
	; (void)fflush(stdout)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);

	fprintf(stderr, "{{{ tests for compare_numbers()...\n")
	; (void)fflush(stderr)
	;
	len	= strlen(codes);
	for (i = 0; i < len; ++i)
	for (j = 0; j < len; ++j)
	for (k = 0; k < len; ++k)
	for (l = 0; l < len; ++l)
	{
		memset(a_str, (int)'\0', sizeof(a_str))
		; memset(b_str, (int)'\0', sizeof(b_str))
		; a_str[0] = b_str[0]	= codes[i]
		; a_str[1] = b_str[1]	= codes[j]
		; a_str[2] = b_str[2]	= codes[k]
		; a_str[3] = b_str[3]	= codes[l]
		; result	= -9;
		; errno		= 0
		; retval	= compare_numbers((const char *)a_str
								, (const char *)b_str
								, &result
								);
		fprintf(stderr
		,	"a_str[]=\"%s\", b_str=\"%s\", "
			"compare_numbers() returns %p, result=%d, errno=%d.\n"
		,				a_str,		b_str
		,							(void *)retval,	result, errno
		); (void)fflush(stderr)
		;
	}

	fprintf(stderr, "\n"); (void)fflush(stderr);

	for (i = 0; i < (sizeof(strs) /sizeof(strs[0])); ++i)
	for (j = 0; j < (sizeof(strs) / sizeof(strs[0])); ++j)
	{
		result		= -9;
		; errno		= 0
		; retval	= compare_numbers(strs[i], strs[j], &result);
		fprintf(stderr
		,	"strs[%lu]=\"%s\", strs[%lu]=\"%s\", "
			"compare_numbers() returns %p, result=%d, errno=%d.\n"
		,		(unsigned long)i, strs[i],		(unsigned long)j, strs[j]
		,							(void *)retval,	result, errno
		); (void)fflush(stderr)
		;
	}
	fprintf(stderr, "}}} finished tests for compare_numbers()\n")
	; (void)fflush(stderr)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);

	fprintf(stderr, "{{{ tests for aggregate_numbers()...\n")
	; (void)fflush(stderr)
	;
	for (i = 0; i < (sizeof(opcodes) / sizeof(opcodes[0])); ++i)
	for (j = 0; j < (sizeof(signs) / sizeof(signs[0])); ++j)
	for (k = 0; k < (sizeof(numerals) / sizeof(numerals[0])); ++k)
	for (l = 0; l < (sizeof(numerals) / sizeof(numerals[0])); ++l)
	for (m = 0; m < (sizeof(signs) / sizeof(signs[0])); ++m)
	for (n = 0; n < (sizeof(numerals) / sizeof(numerals[0])); ++n)
	for (o = 0; o < (sizeof(numerals) / sizeof(numerals[0])); ++o)
	for (h = 0; h < 3; ++h) {
		(void)snprintf(numeral_strs[0], Aggregate_numerals_sz, "%s%s.%s"
				, signs[j], numerals[k], numerals[l]
				)
		; (void)snprintf(numeral_strs[1], Aggregate_numerals_sz, "%s%s.%s"
				, signs[m], numerals[n], numerals[o]
				)
		; numeral_strps[0] = numeral_strs[0]
		; numeral_strps[1] = numeral_strs[1]
		;
		if (h == 0) {
			scale	= (ssize_t)(
							(opcodes[i] == Aggregate_e_opcode_add)?		25
							: (opcodes[i] == Aggregate_e_opcode_sub)?	25
							:											50
							);
		}
		else if (h == 1) {
			scale	= 10;
		}
		else {
			scale	= -1;
		}
		fprintf(stderr
				,	"%s\n"
					"%s\n"
					"%s\t(scale=%ld)\n"
			  	, numeral_strps[0]
				, ((opcodes[i] == Aggregate_e_opcode_add)?		"+"
					: ((opcodes[i] == Aggregate_e_opcode_sub)?	"-"
					: ((opcodes[i] == Aggregate_e_opcode_mul)?	"*"
					:											"/"
					)))
				, numeral_strps[1],	(long)scale
			  ); (void)fflush(stderr);
		retstrp	= aggregate_numbers(
						  opcodes[i]
						, (const char **)numeral_strps
						, N_numerals
						, scale
						);
		if (retstrp == NULL) {
			fprintf(stderr, "***ERROR: errno=%d\n", errno);
		}
		else {
			fprintf(stderr, "%s\n", retstrp);
			(void)destroy_aggregate_numbers(retstrp);
		}
		fprintf(stderr, "--------------------------------------\n")
		; (void)fflush(stderr)
		;
	}

	fprintf(stderr, "\n"); (void)fflush(stderr);

	for (k = 0; k < 2; ++k)
	for (i = 0; i < (sizeof(opcodes)/ sizeof(opcodes[0])); ++i)
	for (j = 0; j < (sizeof(sums) / sizeof(sums[0])); ++j) {
		fprintf(stderr, "%s", sums[0]);
		if (k == 0) {
			scale	= (ssize_t)(
							(opcodes[i] == Aggregate_e_opcode_add)?		25
							: (opcodes[i] == Aggregate_e_opcode_sub)?	25
							:											50
							);
		}
		else {
			scale	= -1;
		}
		if (j > 0) {
			l = 1; do {
				fprintf(stderr, "%s%s"
					, ((opcodes[i] == Aggregate_e_opcode_add)?		"+"
						: ((opcodes[i] == Aggregate_e_opcode_sub)?	"-"
						: ((opcodes[i] == Aggregate_e_opcode_mul)?	"*"
						:											"/"
						)))
					, sums[l]
					);
			} while(l++ < j);
		}
		fprintf(stderr, "\t(scale=%ld)\n", (long)scale)
		; (void)fflush(stderr)
		;
		retstrp	= aggregate_numbers(
						  opcodes[i]
						, (const char **)sums
						, (j + 1)
						, scale
						);
		if (retstrp == NULL) {
			fprintf(stderr, "***ERROR: errno=%d\n", errno);
		}
		else {
			fprintf(stderr, "%s\n", retstrp);
			(void)destroy_aggregate_numbers(retstrp);
		}
		fprintf(stderr, "-------------------------------------\n")
		; (void)fflush(stderr)
		;
	}
	fprintf(stderr, "}}} finished tests for aggregate_numbers()\n")
	; (void)fflush(stderr)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);

	fprintf(stderr, "{{{ integer part formatting test...\n")
	; (void)fflush(stderr)
	;
	for (i = 0; i < (sizeof(opcodes) / sizeof(opcodes[0])); ++i) {
		const char	*numes[3];
		size_t		n_numes;
		char		*opcode_name;

		opcode_name	= ((opcodes[i] == Aggregate_e_opcode_add)?	"+"
						: ((opcodes[i] == Aggregate_e_opcode_sub)?	"-"
						: ((opcodes[i] == Aggregate_e_opcode_mul)?	"*"
						: ((opcodes[i] == Aggregate_e_opcode_div)?	"/"
						:											"?"
						))))
		; n_numes	= 0
		; numes[n_numes++]	= integer_part_formating[0]
		; numes[n_numes++]	= integer_part_formating[1]
		; numes[n_numes++]	= integer_part_formating[2]
		; fprintf(stderr, "%s %s %s %s %s = \n"
				, integer_part_formating[0]
				, opcode_name
				, integer_part_formating[1]
				, opcode_name
				, integer_part_formating[2]
				)
		; (void)fflush(stderr)
		; errno	= 0
		;
		retstrp	= aggregate_numbers(opcodes[i], numes, n_numes, 50);
		if (retstrp == NULL) {
			fprintf(stderr, "***ERROR!***:L%d: errno=%d", __LINE__, errno);
		}
		else {
			fprintf(stderr, "%s (len=%lu: errno=%d)\n"
					, retstrp, (unsigned long)strlen(retstrp), errno
					)
			; (void)destroy_aggregate_numbers(retstrp)
			;
		}
		(void)fflush(stderr);
	}
	fprintf(stderr, "}}} integer part formatting test finished!\n")
	; (void)fflush(stderr)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);

	fprintf(stderr, "{{{ test for absolute_numbers()...\n")
	; (void)fflush(stderr)
	;
	for (i = 0; i < (sizeof(round_tests) / sizeof(round_tests[0])); ++i) {
	for (j = 0; j < 3; ++j) {
		ssize_t	abs_scale	= (ssize_t)(
								(j == 0)?		-1
								: ((j == 1)?	round_tests[i].begin_scale
								:				round_tests[i].end_scale
								));

		fprintf(stderr, "absolute_numbers(\"%s\", %lld) returns "
				,		round_tests[i].str, (long long)abs_scale
				);
		retstrp	= absolute_numbers(round_tests[i].str, abs_scale);
		if (retstrp == NULL) {
			fprintf(stderr, "NULL (L%d: errno=%d)\n", __LINE__, errno);
		}
		else {
			fprintf(stderr, "\"%s\" (L%d: errno=%d)\n"
					,		retstrp, __LINE__, errno
					)
			; (void)destroy_aggregate_numbers(retstrp)
			;
		}
		(void)fflush(stderr);
	}}
	fprintf(stderr, "}}} test for absolute_numbers() finished!\n")
	; (void)fflush(stderr)
	;

	fprintf(stderr, "\n"); (void)fflush(stderr);
	} /* iterate the unit test for gprof profiling... */

	exit(EXIT_SUCCESS); /* exit test program for `abacus()' API */
}
