/***********************************************************************
 * File: conv_line_ctrl.h
 * 	(Extracted from
 *	   "$Id: conv_line_ctrl.h,v 1.3 2009/05/30 19:52:50 maoyam Exp maoyam $")
 *
 * Abstracts:
 *  This C header defines several things for "conv_line_ctrl()" API.
 *
 * Copyright (C) 2006-2007 Maoyam Tokyo, Japan (mailto:maoyam@mail.goo.ne.jp)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *
 **********************************************************************/

#ifndef _CONV_LINE_CTRL_H_ /* { */
#define _CONV_LINE_CTRL_H_
#ifdef __cplusplus
extern "C" {
#endif

/* include system definitions... */
#include <unistd.h>
#include <stddef.h>
#include <stdlib.h>
#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>


/* type definitions to specify the line-control styles */
typedef enum conv_line_ctrl_t {
	  Conv_line_ctrl_to_Unix		/* "\r\n" or "\r" to "\n" */
	, Conv_line_ctrl_Dos_to_Unix	/* "\r\n" to "\n" */
	, Conv_line_ctrl_Mac_to_Unix	/* "\r" to "\n" */
	, Conv_line_ctrl_to_Dos			/* "\r" or "\n" "\r\n" */
	, Conv_line_ctrl_Unix_to_Dos	/* "\n" to "\r\n" */
	, Conv_line_ctrl_Mac_to_Dos		/* "\r" to "\r\n" */
	, Conv_line_ctrl_to_Mac			/* "\r\n" or "\n" "\r" */
	, Conv_line_ctrl_Unix_to_Mac	/* "\n" to "\r" */
	, Conv_line_ctrl_Dos_to_Mac		/* "\r\n" to "\r" */
} Conv_line_ctrl_style_t;


/***********************************************************************
 * This API counts up the line
 *
 * arguments:
 *	src points to the source buffer.
 *	line_ctrl_sequence points to the sequence of line-control.
 *
 * returns:
 *	Non-zero indicates the number of lines.
 *	Zero means several errors occurred.
 *	(if this returns zero, errno indicates the details.
 *		EINVAL  (22)	: invalid arguments
 **********************************************************************/
extern size_t count_lines(char *src, char *line_ctrl_sequence);

/***********************************************************************
 * This API converts the line control sequences in a string
 *
 * arguments:
 *	style specifies the line-control styles.
 *	dst points to the destiny buffer.
 *	dst_sz indicates the size of destiny buffer.
 *	src points to the source buffer.
 *
 * returns:
 *	NULL means several errors are occurred.  Otherwise, this returns
 *	the address of destiny buffer.
 *	(if this returns NULL, errno indicates the details.
 *		EINVAL  (22)	: invalid arguments
 *		ENOBUFS (105)	: destiny buffer is insufficient.
 **********************************************************************/
extern char *conv_line_ctrl(Conv_line_ctrl_style_t	style
							, char					*dst
							, size_t				dst_sz
							, char					*src
							);

#ifdef __cplusplus
}
#endif
#endif /* } !define(_CONV_LINE_CTRL_H_) */
